# JUNGFRAU Calibration

This project details the JUNGFRAU calibration procedure. 

## Copper fluorescence
Provides an absolute calibration of high gain. Copper fluorescence is used to illuminate the sensor. The resulting noise and Kalpha peaks are fit with a gaussian and charge sharing model. The gain for 8 keV is the distance between the two peaks.

Scripts are:
- *CuFluo_exposure.sh* to perform the exposure in the X-ray box
- *CuFluo_analysis.cpp* to measure the peak-to-peak distance
- *CuFluo_Ka_KaKb_Comp.cpp* to quantify the difference between different fit models

## Direct beam
Provides a relative calibration between high and medium gains. The current in the X-ray box is varied to measure the response of high and medium gains. The ratio of the two gains is the ratio of fitted gradients.

Scripts are:
- *DB_scan.sh* to perform the scan
- *DB_analysis.cpp* to calculate the response, fit the gradients and measure the ratio

## Backplane pulsing
Also provides a relative calibration between high and medium gains. A seperate circuit board is used to pulse the voltage on the backplane of the sensor. By varying the size of the pulse, the response of high and medium gains can be measured and fit with linear functions. The ratio of the two gains is the ratio of fitted gradients.

Scripts are:
- *BP_scan.sh* to perform the scan
- *BP_analysis.cpp* to calculate the response, fit the gradients and measure the ratio
- *BP_DB_Comp.cpp* to quantify the difference between the two methods

## Current source
Provides a relative calibration between medium and low gains. An internal current source is used to inject charge into the ASIC one column at a time. By varying the integration time, the amound of collected charge is varied and the response of medium and low gain can be measured. The ratio of the two gains is the ratio of fitted gradients.

Scripts are:
- *CS_scan.sh* to perform the scan
- *CS_analysis.cpp* to calculate the response, fit the gradients and measure the ratio
- *CS_BaselineTest.cpp* to check that increasing the integratio time doesn't change the baseline

## Full dynamic range calibration
The full dynamic range can be calibrated by combining information from the various measurements.

- High gain = peak-to-peak difference [ADU] / 8 [keV]
- Medium gain = high gain [ADU / keV] * medium gain / high gain
- Low gain = high gain [ADU / keV] * medium gain / high gain * low gain / medium gain

Scripts are:
- *JFMC_CalibWriter.cpp* to perform the calibration

## References
- Calibration status and plans for the charge integrating JUNGFRAU pixel detector for SwissFEL, S. Redford et al 2016 JINST 11 C11013
- First full dynamic range calibration of the JUNGFRAU photon detector, S. Redford et al 2018 JINST 13 C01027